@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2010       Michiel D. Nauta
-# Copyright (C) 2010       Nick Hall
+# Copyright (C) 2010,2017  Nick Hall
 # Copyright (C) 2011       Tim G L Lyons
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -32,11 +32,10 @@ Family object for Gramps.
 #-------------------------------------------------------------------------
 from warnings import warn
 import logging
-LOG = logging.getLogger(".citation")
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #-------------------------------------------------------------------------
 from .primaryobj import PrimaryObject
@@ -50,7 +49,10 @@ from .tagbase import TagBase
 from .childref import ChildRef
 from .familyreltype import FamilyRelType
 from .const import IDENTICAL, EQUAL, DIFFERENT
-from .handle import Handle
+from ..const import GRAMPS_LOCALE as glocale
+_ = glocale.translation.gettext
+
+LOG = logging.getLogger(".citation")
 
 #-------------------------------------------------------------------------
 #
@@ -63,7 +65,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
     The Family record is the Gramps in-memory representation of the
     relationships between people. It contains all the information
     related to the relationship.
-    
+
     Family objects are usually created in one of two ways.
 
     1. Creating a new Family object, which is then initialized and
@@ -78,9 +80,9 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def __init__(self):
         """
-        Create a new Family instance. 
-        
-        After initialization, most data items have empty or null values, 
+        Create a new Family instance.
+
+        After initialization, most data items have empty or null values,
         including the database handle.
         """
         PrimaryObject.__init__(self)
@@ -99,9 +101,9 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
     def serialize(self):
         """
         Convert the data held in the event to a Python tuple that
-        represents all the data elements. 
-        
-        This method is used to convert the object into a form that can easily 
+        represents all the data elements.
+
+        This method is used to convert the object into a form that can easily
         be saved to a database.
 
         These elements may be primitive Python types (string, integers),
@@ -126,66 +128,68 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
                 NoteBase.serialize(self),
                 self.change, TagBase.serialize(self), self.private)
 
-    def to_struct(self):
+    @classmethod
+    def get_schema(cls):
         """
-        Convert the data held in this object to a structure (eg,
-        struct) that represents all the data elements.
-        
-        This method is used to recursively convert the object into a
-        self-documenting form that can easily be used for various
-        purposes, including diffs and queries.
+        Returns the JSON Schema for this class.
 
-        These structures may be primitive Python types (string,
-        integer, boolean, etc.) or complex Python types (lists,
-        tuples, or dicts). If the return type is a dict, then the keys
-        of the dict match the fieldname of the object. If the return
-        struct (or value of a dict key) is a list, then it is a list
-        of structs. Otherwise, the struct is just the value of the
-        attribute.
-
-        :returns: Returns a struct containing the data of the object.
+        :returns: Returns a dict containing the schema.
         :rtype: dict
         """
-        return {"_class": "Family",
-                "handle": Handle("Family", self.handle), 
-                "gramps_id": self.gramps_id, 
-                "father_handle": Handle("Person", self.father_handle),
-                "mother_handle": Handle("Person", self.mother_handle),
-                "child_ref_list": [cr.to_struct() for cr in self.child_ref_list],
-                "type": self.type.to_struct(),
-                "event_ref_list": [er.to_struct() for er in self.event_ref_list],
-                "media_list": MediaBase.to_struct(self),
-                "attribute_list": AttributeBase.to_struct(self),
-                "lds_ord_list": LdsOrdBase.to_struct(self),
-                "citation_list": CitationBase.to_struct(self),
-                "note_list": NoteBase.to_struct(self),
-                "change": self.change, 
-                "tag_list": TagBase.to_struct(self), 
-                "private": self.private}
-
-    @classmethod
-    def from_struct(cls, struct):
-        """
-        Given a struct data representation, return a serialized object.
-
-        :returns: Returns a serialized object
-        """
-        default = Family()
-        return (Handle.from_struct(struct.get("handle", default.handle)),
-                struct.get("gramps_id", default.gramps_id),
-                Handle.from_struct(struct.get("father_handle", default.father_handle)),
-                Handle.from_struct(struct.get("mother_handle", default.mother_handle)),
-                [ChildRef.from_struct(cr) for cr in struct.get("child_ref_list", default.child_ref_list)],
-                FamilyRelType.from_struct(struct.get("type", {})),
-                [EventRef.from_struct(er) for er in struct.get("event_ref_list", default.event_ref_list)],
-                MediaBase.from_struct(struct.get("media_list", default.media_list)),
-                AttributeBase.from_struct(struct.get("attribute_list", default.attribute_list)),
-                LdsOrdBase.from_struct(struct.get("lds_ord_list", default.lds_ord_list)),
-                CitationBase.from_struct(struct.get("citation_list", default.citation_list)),
-                NoteBase.from_struct(struct.get("note_list", default.note_list)),
-                struct.get("change", default.change), 
-                TagBase.from_struct(struct.get("tag_list", default.tag_list)), 
-                struct.get("private", default.private))
+        from .mediaref import MediaRef
+        from .ldsord import LdsOrd
+        from .childref import ChildRef
+        from .attribute import Attribute
+        return {
+            "type": "object",
+            "title": _("Family"),
+            "properties": {
+                "_class": {"enum": [cls.__name__]},
+                "handle": {"type": "string",
+                           "maxLength": 50,
+                           "title": _("Handle")},
+                "gramps_id": {"type": "string",
+                              "title": _("Gramps ID")},
+                "father_handle": {"type": ["string", "null"],
+                                  "maxLength": 50,
+                                  "title": _("Father")},
+                "mother_handle": {"type": ["string", "null"],
+                                  "maxLength": 50,
+                                  "title": _("Mother")},
+                "child_ref_list": {"type": "array",
+                                   "items": ChildRef.get_schema(),
+                                   "title": _("Children")},
+                "type": FamilyRelType.get_schema(),
+                "event_ref_list": {"type": "array",
+                                   "items": EventRef.get_schema(),
+                                   "title": _("Events")},
+                "media_list": {"type": "array",
+                               "items": MediaRef.get_schema(),
+                               "title": _("Media")},
+                "attribute_list": {"type": "array",
+                                   "items": Attribute.get_schema(),
+                                   "title": _("Attributes")},
+                "lds_ord_list": {"type": "array",
+                                 "items": LdsOrd.get_schema(),
+                                 "title": _("LDS ordinances")},
+                "citation_list": {"type": "array",
+                                  "items": {"type": "string",
+                                            "maxLength": 50},
+                                  "title": _("Citations")},
+                "note_list": {"type": "array",
+                              "items": {"type": "string",
+                                        "maxLength": 50},
+                              "title": _("Notes")},
+                "change": {"type": "integer",
+                           "title": _("Last changed")},
+                "tag_list": {"type": "array",
+                             "items": {"type": "string",
+                                       "maxLength": 50},
+                             "title": _("Tags")},
+                "private": {"type": "boolean",
+                            "title": _("Private")}
+            }
+        }
 
     def unserialize(self, data):
         """
@@ -213,14 +217,14 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def _has_handle_reference(self, classname, handle):
         """
-        Return True if the object has reference to a given handle of given 
+        Return True if the object has reference to a given handle of given
         primary object type.
-        
+
         :param classname: The name of the primary object class.
         :type classname: str
         :param handle: The handle to be checked.
         :type handle: str
-        :returns: Returns whether the object has reference to this handle of 
+        :returns: Returns whether the object has reference to this handle of
                   this object type.
         :rtype: bool
         """
@@ -230,7 +234,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
             return handle in ([ref.ref for ref in self.child_ref_list]
                               + [self.father_handle, self.mother_handle])
         elif classname == 'Place':
-            return handle in [ x.place for x in self.lds_ord_list ]
+            return handle in [x.place for x in self.lds_ord_list]
         return False
 
     def _remove_handle_references(self, classname, handle_list):
@@ -244,20 +248,20 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         if classname == 'Event':
             new_list = [ref for ref in self.event_ref_list
-                            if ref.ref not in handle_list]
+                        if ref.ref not in handle_list]
             self.event_ref_list = new_list
         elif classname == 'Person':
             new_list = [ref for ref in self.child_ref_list
-                            if ref.ref not in handle_list]
+                        if ref.ref not in handle_list]
             self.child_ref_list = new_list
             if self.father_handle in handle_list:
                 self.father_handle = None
             if self.mother_handle in handle_list:
                 self.mother_handle = None
         elif classname == 'Place':
-            for x in self.lds_ord_list:
-                if x.place in handle_list:
-                    x.place = None
+            for lds_ord in self.lds_ord_list:
+                if lds_ord.place in handle_list:
+                    lds_ord.place = None
 
     def _replace_handle_reference(self, classname, old_handle, new_handle):
         """
@@ -271,7 +275,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         :type new_handle: str
         """
         if classname == 'Event':
-            refs_list = [ ref.ref for ref in self.event_ref_list ]
+            refs_list = [ref.ref for ref in self.event_ref_list]
             new_ref = None
             if new_handle in refs_list:
                 new_ref = self.event_ref_list[refs_list.index(new_handle)]
@@ -289,7 +293,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
                         self.event_ref_list.pop(idx)
                         refs_list.pop(idx)
         elif classname == 'Person':
-            refs_list = [ ref.ref for ref in self.child_ref_list ]
+            refs_list = [ref.ref for ref in self.child_ref_list]
             new_ref = None
             if new_handle in refs_list:
                 new_ref = self.child_ref_list[refs_list.index(new_handle)]
@@ -311,9 +315,9 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
             if self.mother_handle == old_handle:
                 self.mother_handle = new_handle
         elif classname == 'Place':
-            for x in self.lds_ord_list:
-                if x.place == old_handle:
-                    x.place = new_handle
+            for lds_ord in self.lds_ord_list:
+                if lds_ord.place == old_handle:
+                    lds_ord.place = new_handle
 
     def get_text_data_list(self):
         """
@@ -338,7 +342,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of child secondary objects that may refer citations.
 
-        :returns: Returns the list of child secondary child objects that may 
+        :returns: Returns the list of child secondary child objects that may
                   refer citations.
         :rtype: list
         """
@@ -350,7 +354,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of child secondary objects that may refer notes.
 
-        :returns: Returns the list of child secondary child objects that may 
+        :returns: Returns the list of child secondary child objects that may
                   refer notes.
         :rtype: list
         """
@@ -363,7 +367,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of (classname, handle) tuples for all directly
         referenced primary objects.
-        
+
         :returns: List of (classname, handle) tuples for referenced objects.
         :rtype: list
         """
@@ -378,9 +382,9 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def get_handle_referents(self):
         """
-        Return the list of child objects which may, directly or through their 
+        Return the list of child objects which may, directly or through their
         children, reference primary objects..
-        
+
         :returns: Returns the list of objects referencing primary objects.
         :rtype: list
         """
@@ -411,14 +415,14 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
     def set_relationship(self, relationship_type):
         """
         Set the relationship type between the people identified as the
-        father and mother in the relationship. 
-        
-        The type is a tuple whose first item is an integer constant and whose 
+        father and mother in the relationship.
+
+        The type is a tuple whose first item is an integer constant and whose
         second item is the string. The valid values are:
 
-        =========================  ============================================ 
+        =========================  ============================================
         Type                       Description
-        =========================  ============================================ 
+        =========================  ============================================
         FamilyRelType.MARRIED      indicates a legally recognized married
                                    relationship between two individuals. This
                                    may be either an opposite or a same sex
@@ -434,7 +438,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         FamilyRelType.CUSTOM       indicates that the type of relationship
                                    between the two individuals does not match
                                    any of the other types.
-        =========================  ============================================ 
+        =========================  ============================================
 
         :param relationship_type: (int,str) tuple of the relationship type
                between the father and mother of the relationship.
@@ -451,10 +455,10 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def set_father_handle(self, person_handle):
         """
-        Set the database handle for :class:`~.person.Person` that corresponds 
-        to male of the relationship. 
-        
-        For a same sex relationship, this can represent either of people 
+        Set the database handle for :class:`~.person.Person` that corresponds
+        to male of the relationship.
+
+        For a same sex relationship, this can represent either of people
         involved in the relationship.
 
         :param person_handle: :class:`~.person.Person` database handle
@@ -464,7 +468,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def get_father_handle(self):
         """
-        Return the database handle of the :class:`~.person.Person` identified 
+        Return the database handle of the :class:`~.person.Person` identified
         as the father of the Family.
 
         :returns: :class:`~.person.Person` database handle
@@ -474,10 +478,10 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def set_mother_handle(self, person_handle):
         """
-        Set the database handle for :class:`~.person.Person` that corresponds 
-        to male of the relationship. 
-        
-        For a same sex relationship, this can represent either of people 
+        Set the database handle for :class:`~.person.Person` that corresponds
+        to male of the relationship.
+
+        For a same sex relationship, this can represent either of people
         involved in the relationship.
 
         :param person_handle: :class:`~.person.Person` database handle
@@ -487,7 +491,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def get_mother_handle(self):
         """
-        Return the database handle of the :class:`~.person.Person` identified 
+        Return the database handle of the :class:`~.person.Person` identified
         as the mother of the Family.
 
         :returns: :class:`~.person.Person` database handle
@@ -506,7 +510,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         if not isinstance(child_ref, ChildRef):
             raise ValueError("expecting ChildRef instance")
         self.child_ref_list.append(child_ref)
-            
+
     def remove_child_ref(self, child_ref):
         """
         Remove the database handle for :class:`~.person.Person` to the Family's
@@ -521,7 +525,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         if not isinstance(child_ref, ChildRef):
             raise ValueError("expecting ChildRef instance")
         new_list = [ref for ref in self.child_ref_list
-                    if ref.ref != child_ref.ref ]
+                    if ref.ref != child_ref.ref]
         self.child_ref_list = new_list
 
     def remove_child_handle(self, child_handle):
@@ -536,7 +540,7 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         :rtype: bool
         """
         new_list = [ref for ref in self.child_ref_list
-                    if ref.ref != child_handle ]
+                    if ref.ref != child_handle]
         self.child_ref_list = new_list
 
     def get_child_ref_list(self):
@@ -584,10 +588,10 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Add the :class:`~.eventref.EventRef` to the Family instance's
         :class:`~.eventref.EventRef` list.
-        
+
         This is accomplished by assigning the :class:`~.eventref.EventRef` for
         the valid :class:`~.event.Event` in the current database.
-        
+
         :param event_ref: the :class:`~.eventref.EventRef` to be added to the
                           Person's :class:`~.eventref.EventRef` list.
         :type event_ref: EventRef
@@ -596,28 +600,28 @@ class Family(CitationBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
             raise ValueError("Expecting EventRef instance")
         self.event_ref_list.append(event_ref)
 
-    def get_event_list(self) :
-        warn( "Use get_event_ref_list instead of get_event_list",
-              DeprecationWarning, 2)
+    def get_event_list(self):
+        warn("Use get_event_ref_list instead of get_event_list",
+             DeprecationWarning, 2)
         # Wrapper for old API
         # remove when transitition done.
         event_handle_list = []
         for event_ref in self.get_event_ref_list():
-            event_handle_list.append( event_ref.get_reference_handle())
+            event_handle_list.append(event_ref.get_reference_handle())
         return event_handle_list
 
-    def get_event_ref_list(self) :
+    def get_event_ref_list(self):
         """
         Return the list of :class:`~.eventref.EventRef` objects associated with
         :class:`~.event.Event` instances.
 
-        :returns: Returns the list of :class:`~.eventref.EventRef` objects 
+        :returns: Returns the list of :class:`~.eventref.EventRef` objects
                   associated with the Family instance.
         :rtype: list
         """
         return self.event_ref_list
 
-    def set_event_ref_list(self, event_ref_list) :
+    def set_event_ref_list(self, event_ref_list):
         """
         Set the Family instance's :class:`~.eventref.EventRef` list to the
         passed list.
