@@ -8,14 +8,14 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -27,7 +27,7 @@ Provide the base class for GRAMPS' DataView classes
 # python modules
 #
 #----------------------------------------------------------------
-import sys
+from abc import ABCMeta, abstractmethod
 import logging
 _LOG = logging.getLogger('.pageview')
 
@@ -43,7 +43,7 @@ _ = glocale.translation.gettext
 
 #----------------------------------------------------------------
 #
-# GRAMPS 
+# Gramps
 #
 #----------------------------------------------------------------
 from gramps.gen.errors import WindowActiveError
@@ -51,18 +51,19 @@ from ..dbguielement import DbGUIElement
 from ..widgets.grampletbar import GrampletBar
 from ..configure import ConfigureDialog
 from gramps.gen.config import config
+from ..uimanager import ActionGroup
 
 #------------------------------------------------------------------------------
 #
 # PageView
 #
 #------------------------------------------------------------------------------
-class PageView(DbGUIElement):
+class PageView(DbGUIElement, metaclass=ABCMeta):
     """
-    The PageView class is the base class for all Data Views in GRAMPS.  All 
+    The PageView class is the base class for all Data Views in Gramps.  All
     Views should derive from this class. The ViewManager understands the public
     interface of this class
-    
+
     The following attributes are available
     ..attribute:: active
       is the view active at the moment (visible in Gramps as the main view)
@@ -70,11 +71,11 @@ class PageView(DbGUIElement):
       bool, is the view current with the database or not. A dirty view triggers
       a rebuild when it becomes active
     ..attribute:: _dirty_on_change_inactive
-      read/write bool by inheriting classes. 
+      read/write bool by inheriting classes.
       Views can behave two ways to signals:
-      1. if the view is inactive, set it dirty, and rebuild the view when 
+      1. if the view is inactive, set it dirty, and rebuild the view when
           it becomes active. In this case, this method returns True
-      2. if the view is inactive, try to stay in sync with database. Only 
+      2. if the view is inactive, try to stay in sync with database. Only
          rebuild or other large changes make view dirty
     ..attribute:: title
       title of the view
@@ -96,26 +97,25 @@ class PageView(DbGUIElement):
         self.uistate = uistate
         self.action_list = []
         self.action_toggle_list = []
-        self.action_toolmenu_list = []
-        self.action_toolmenu = {} #easy access to toolmenuaction and proxies
         self.action_group = None
         self.additional_action_groups = []
         self.additional_uis = []
-        self.ui_def = '''<ui>
-          <menubar name="MenuBar">
-            <menu action="ViewMenu">
-              <placeholder name="Bars">
-                <menuitem action="Sidebar"/>
-                <menuitem action="Bottombar"/>
-              </placeholder>
-            </menu>
-          </menubar>
-        </ui>'''
+        self.ui_def = ['''
+          <placeholder id="Bars">
+            <item>
+              <attribute name="action">win.Sidebar</attribute>
+              <attribute name="label" translatable="yes">_Sidebar</attribute>
+            </item>
+             <item>
+              <attribute name="action">win.Bottombar</attribute>
+              <attribute name="label" translatable="yes">_Bottombar</attribute>
+            </item>
+          </placeholder>
+            ''']
         self.dirty = True
         self.active = False
         self._dirty_on_change_inactive = True
-        self.func_list = {}
-        
+
         if isinstance(self.pdata.category, tuple):
             self.category, self.translated_category = self.pdata.category
         else:
@@ -125,11 +125,11 @@ class PageView(DbGUIElement):
         self.dbstate.connect('no-database', self.disable_action_group)
         self.dbstate.connect('database-changed', self.enable_action_group)
         self.uistate.window.connect("key-press-event", self.key_press_handler)
-        
+
         self.model = None
         self.selection = None
         self.handle_col = 0
-        
+
         self._config = None
         self.init_config()
 
@@ -137,6 +137,7 @@ class PageView(DbGUIElement):
         self.top = None
         self.sidebar = None
         self.bottombar = None
+        self.widget = None
 
         DbGUIElement.__init__(self, dbstate.db)
 
@@ -152,40 +153,73 @@ class PageView(DbGUIElement):
         self.bottombar = GrampletBar(self.dbstate, self.uistate, self,
                                      self.ident + "_bottombar",
                                      defaults[1])
-        hpane = Gtk.HPaned()
-        vpane = Gtk.VPaned()
+        hpane = Gtk.Paned()
+        vpane = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
         hpane.pack1(vpane, resize=True, shrink=False)
-        hpane.pack2(self.sidebar, resize=False, shrink=True)
+        hpane.pack2(self.sidebar, resize=False, shrink=False)
         hpane.show()
         vpane.show()
 
-        widget = self.build_widget()
-        widget.show_all()
-        vpane.pack1(widget, resize=True, shrink=False)
+        self.widget = self.build_widget()
+        self.widget.show_all()
+        self.widget.set_name('view')
+        vpane.pack1(self.widget, resize=True, shrink=False)
         vpane.pack2(self.bottombar, resize=False, shrink=True)
+        self._setup_slider_config(vpane, 'vpane.slider-position')
 
         self.sidebar_toggled(self.sidebar.get_property('visible'))
+        self.hpane_sig = hpane.connect("draw", self.set_page_slider)
 
         return hpane
 
-    def __sidebar_toggled(self, action):
+    def set_page_slider(self, widget, dummy):
+        """ Setup slider.  We have the page realized at this point. """
+        widget.disconnect(self.hpane_sig)
+        # get current width of pane
+        width = widget.get_allocated_width()
+        # default will use natural size for sidebar until it gets to 400 pix
+        side_ch = self.sidebar.get_children()  # Gtk Notebook
+        try:
+            vp_ch = side_ch[0].get_children()  # Gtk Viewport child
+            ch_width = vp_ch[0].get_preferred_width()[0] + 3
+        except AttributeError:
+            ch_width = 300  # needed if no Gramplet installed
+        pos = width - min(ch_width, 400)
+        self._setup_slider_config(widget, 'hpane.slider-position',
+                                  position=pos)
+
+    def _setup_slider_config(self, widget, setting, position=-1):
+        """
+        Setup the slider configuration setting.
+        """
+        self._config.register(setting, position)
+        widget.set_position(self._config.get(setting))
+        widget.connect('notify::position', self._position_changed, setting)
+
+    def _position_changed(self, widget, position, setting):
+        """
+        Called when the position property of the pane is changed.
+        """
+        self._config.set(setting, widget.get_position())
+
+    def __sidebar_toggled(self, action, value):
         """
         Called when the sidebar is toggled.
         """
-        active = action.get_active()
-        if active:
+        action.set_state(value)  # change GUI
+        if value.get_boolean():
             self.sidebar.show()
             self.sidebar_toggled(True)
         else:
             self.sidebar.hide()
             self.sidebar_toggled(False)
 
-    def __bottombar_toggled(self, action):
+    def __bottombar_toggled(self, action, value):
         """
         Called when the bottombar is toggled.
         """
-        active = action.get_active()
-        if active:
+        action.set_state(value)  # change GUI
+        if value.get_boolean():
             self.bottombar.show()
         else:
             self.bottombar.hide()
@@ -202,7 +236,7 @@ class PageView(DbGUIElement):
         Returns a 2-tuple.  The first element is a tuple of sidebar gramplets
         and the second element is a tuple of bottombar gramplets.
 
-        Views should override this method to define default gramplets. 
+        Views should override this method to define default gramplets.
         """
         return ((), ())
 
@@ -223,10 +257,7 @@ class PageView(DbGUIElement):
         creates it. The copy is handled through the drag and drop
         system.
         """
-        if sys.version_info[0] < 3:
-            import cPickle as pickle
-        else:
-            import pickle
+        import pickle
         from ..clipboard import ClipboardWindow, obj2target
         handled = False
         for handle in handles:
@@ -241,13 +272,13 @@ class PageView(DbGUIElement):
             # Construct a drop:
             drag_type = obj2target(objclass)
             if drag_type:
-                class Selection(object):
+                class Selection:
                     def __init__(self, data):
                         self.data = data
                     def get_data(self):
                         return self.data
-                class Context(object):
-                    targets = [drag_type]
+                class Context:
+                    targets = [Gdk.atom_intern(drag_type.name(), False)]
                     action = 1
                     def list_targets(self):
                         return Context.targets
@@ -265,7 +296,7 @@ class PageView(DbGUIElement):
                 handled = True
         return handled
 
-    def call_paste(self): 
+    def call_paste(self):
         """
         This code is called on Control+V in a navigation view. If the
         copy can be handled, it returns true, otherwise false.
@@ -282,18 +313,12 @@ class PageView(DbGUIElement):
             return True
         return False
 
-    def call_function(self, key):
-        """
-        Calls the function associated with the key value
-        """
-        self.func_list.get(key)()
-
     def post(self):
         """
         Called after a page is created.
         """
         pass
-    
+
     def set_active(self):
         """
         Called with the PageView is set as active. If the page is "dirty",
@@ -302,11 +327,14 @@ class PageView(DbGUIElement):
         self.sidebar.set_active()
         self.bottombar.set_active()
         self.active = True
+        new_title = "%s - %s - Gramps" % (self.dbstate.db.get_dbname(),
+                                      self.get_title())
+        self.uistate.window.set_title(new_title)
         if self.dirty:
             self.uistate.set_busy_cursor(True)
             self.build_tree()
             self.uistate.set_busy_cursor(False)
-            
+
     def set_inactive(self):
         """
         Marks page as being inactive (not currently displayed)
@@ -315,13 +343,13 @@ class PageView(DbGUIElement):
         self.bottombar.set_inactive()
         self.active = False
 
+    @abstractmethod
     def build_tree(self):
         """
         Rebuilds the current display. This must be overridden by the derived
         class.
         """
-        raise NotImplementedError
-    
+
     def ui_definition(self):
         """
         returns the XML UI definition for the UIManager
@@ -340,28 +368,29 @@ class PageView(DbGUIElement):
         Turns off the visibility of the View's action group, if defined
         """
         if self.action_group:
-            self.action_group.set_visible(False)
+            self.uistate.uimanager.set_actions_visible(self.action_group,
+                                                       False)
 
     def enable_action_group(self, obj):
         """
         Turns on the visibility of the View's action group, if defined
         """
         if self.action_group:
-            self.action_group.set_visible(True)
+            self.uistate.uimanager.set_actions_visible(self.action_group, True)
 
     def get_stock(self):
         """
-        Return image associated with the view category, which is used for the 
+        Return image associated with the view category, which is used for the
         icon for the button.
         """
-        return Gtk.STOCK_MISSING_IMAGE
+        return 'image-missing'
 
     def get_viewtype_stock(self):
         """
         Return immage associated with the viewtype inside a view category, it
         will be used for the icon on the button to select view in the category
         """
-        return Gtk.STOCK_MISSING_IMAGE
+        return 'image-missing'
 
     def get_title(self):
         """
@@ -392,72 +421,58 @@ class PageView(DbGUIElement):
             self.top = self.build_interface()
         return self.top
 
+    @abstractmethod
     def build_widget(self):
         """
         Builds the container widget for the main view pane. Must be overridden
         by the base class. Returns a gtk container widget.
         """
-        raise NotImplementedError
 
     def define_actions(self):
         """
         Defines the UIManager actions. Called by the ViewManager to set up the
-        View. The user typically defines self.action_list and 
-        self.action_toggle_list in this function. 
+        View. The user typically defines self.action_list and
+        self.action_toggle_list in this function.
         """
-        self._add_toggle_action('Sidebar', None, _('_Sidebar'), 
-             "<shift><PRIMARY>R", None, self.__sidebar_toggled,
-             self.sidebar.get_property('visible'))
-        self._add_toggle_action('Bottombar', None, _('_Bottombar'), 
-             "<shift><PRIMARY>B", None, self.__bottombar_toggled,
-             self.bottombar.get_property('visible'))
+        self._add_toggle_action('Sidebar', self.__sidebar_toggled,
+            '<shift><PRIMARY>R', self.sidebar.get_property('visible'))
+        self._add_toggle_action('Bottombar', self.__bottombar_toggled,
+            '<shift><PRIMARY>B', self.bottombar.get_property('visible'))
 
     def __build_action_group(self):
         """
         Create an UIManager ActionGroup from the values in self.action_list
-        and self.action_toggle_list. The user should define these in 
+        and self.action_toggle_list. The user should define these in
         self.define_actions
         """
-        self.action_group = Gtk.ActionGroup(name=self.title)
+        self.action_group = ActionGroup(name=self.title)
         if len(self.action_list) > 0:
             self.action_group.add_actions(self.action_list)
         if len(self.action_toggle_list) > 0:
-            self.action_group.add_toggle_actions(self.action_toggle_list)
+            self.action_group.add_actions(self.action_toggle_list)
 
-    def _add_action(self, name, stock_icon, label, accel=None, tip=None, 
-                   callback=None):
+    def _add_action(self, name, callback=None, accel=None):
         """
-        Add an action to the action list for the current view. 
+        Add an action to the action list for the current view.
         """
-        self.action_list.append((name, stock_icon, label, accel, tip, 
-                                 callback))
+        self.action_list.append((name, callback, accel))
 
-    def _add_toggle_action(self, name, stock_icon, label, accel=None, 
-                           tip=None, callback=None, value=False):
+    def _add_toggle_action(self, name, callback=None, accel= None, value=False):
         """
-        Add a toggle action to the action list for the current view. 
+        Add a toggle action to the action list for the current view.
         """
-        self.action_toggle_list.append((name, stock_icon, label, accel, 
-                                        tip, callback, value))
-    
-    def _add_toolmenu_action(self, name, label, tooltip, callback, 
-                             arrowtooltip):
-        """
-        Add a menu action to the action list for the current view. 
-        """
-        self.action_toolmenu_list.append((name, label, tooltip, callback,
-                                          arrowtooltip))
+        self.action_toggle_list.append((name, callback, accel, value))
 
     def get_actions(self):
         """
         Return the actions that should be used for the view. This includes the
-        standard action group (which handles the main toolbar), along with 
+        standard action group (which handles the main toolbar), along with
         additional action groups.
 
-        If the action group is not defined, we build it the first time. This 
+        If the action group is not defined, we build it the first time. This
         allows us to delay the intialization until it is really needed.
 
-        The ViewManager uses this function to extract the actions to install 
+        The ViewManager uses this function to extract the actions to install
         into the UIManager.
         """
         if not self.action_group:
@@ -466,7 +481,7 @@ class PageView(DbGUIElement):
 
     def _add_action_group(self, group):
         """
-        Allows additional action groups to be added to the view. 
+        Allows additional action groups to be added to the view.
         """
         self.additional_action_groups.append(group)
 
@@ -476,40 +491,6 @@ class PageView(DbGUIElement):
         """
         self.uistate.clear_filter_results()
 
-    def edit(self, obj):
-        """
-        Template function to allow the editing of the selected object
-        """
-        raise NotImplementedError
-
-    def remove(self, handle):
-        """
-        Template function to allow the removal of an object by its handle
-        """
-        raise NotImplementedError
-
-    def remove_object_from_handle(self, handle):
-        """
-        Template function to allow the removal of an object by its handle
-        """
-        raise NotImplementedError
-
-    def add(self, obj):
-        """
-        Template function to allow the adding of a new object
-        """
-        raise NotImplementedError
-    
-    def _key_press(self, obj, event):
-        """
-        Define the action for a key press event
-        """
-        # TODO: This is never used? (replaced in ListView)
-        if event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
-            self.edit(obj)
-            return True
-        return False
-
     def on_delete(self):
         """
         Method called on shutdown. Data views should put code here
@@ -521,21 +502,21 @@ class PageView(DbGUIElement):
 
     def init_config(self):
         """
-        If you need a view with a config, then call this method in the 
-        build_widget or __init__ method. It will set up a config file for the 
-        view, and use CONFIGSETTINGS to set the config defaults. 
+        If you need a view with a config, then call this method in the
+        build_widget or __init__ method. It will set up a config file for the
+        view, and use CONFIGSETTINGS to set the config defaults.
         The config is later accessbile via self._config
-        So you can do 
+        So you can do
         self._config.get("section.variable1")
         self._config.set("section.variable1", value)
         self._config.save()
-        
-        CONFIGSETTINGS should be a list with tuples like 
+
+        CONFIGSETTINGS should be a list with tuples like
         ("section.variable1", value)
         """
         if self._config:
             return
-        self._config = config.register_manager(self.ident, 
+        self._config = config.register_manager(self.ident,
                                                use_config_path=True)
         for section, value in self.CONFIGSETTINGS:
             self._config.register(section, value)
@@ -567,19 +548,19 @@ class PageView(DbGUIElement):
 
     def _get_configure_page_funcs(self):
         """
-        Return a list of functions that create gtk elements to use in the 
+        Return a list of functions that create gtk elements to use in the
         notebook pages of the Configure view
-        
+
         :return: list of functions
         """
-        raise NotImplementedError
+        return []
 
     def configure(self):
         """
         Open the configure dialog for the view.
         """
         title = _("Configure %(cat)s - %(view)s") % \
-                        {'cat': self.get_translated_category(), 
+                        {'cat': self.get_translated_category(),
                          'view': self.get_title()}
 
         if self.can_configure():
@@ -592,10 +573,10 @@ class PageView(DbGUIElement):
             config_funcs += self.bottombar.get_config_funcs()
 
         try:
-            ViewConfigureDialog(self.uistate, self.dbstate, 
+            ViewConfigureDialog(self.uistate, self.dbstate,
                             config_funcs,
                             self, self._config, dialogtitle=title,
-                            ident=_("%(cat)s - %(view)s") % 
+                            ident=_("%(cat)s - %(view)s") %
                                     {'cat': self.get_translated_category(),
                                      'view': self.get_title()})
         except WindowActiveError:
@@ -612,7 +593,8 @@ class ViewConfigureDialog(ConfigureDialog):
         ConfigureDialog.__init__(self, uistate, dbstate, configure_page_funcs,
                                  configobj, configmanager,
                                  dialogtitle=dialogtitle, on_close=on_close)
-        
+        self.setup_configs('interface.viewconfiguredialog', 420, 500)
+
     def build_menu_names(self, obj):
         return (_('Configure %s View') % self.ident, None)
 
@@ -624,9 +606,9 @@ class DummyPage(PageView):
         self.msg = msg1
         self.msg2 = msg2
         PageView.__init__(self, title, pdata, dbstate, uistate)
-    
+
     def build_widget(self):
-        box = Gtk.VBox(homogeneous=False, spacing=1)
+        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=1)
         #top widget at the top
         box.pack_start(Gtk.Label(label=_('View %(name)s: %(msg)s') % {
                 'name': self.title,
