@@ -8,7 +8,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -36,8 +36,14 @@ _LOG = logging.getLogger(".widgets.interactivesearch")
 # GTK modules
 #
 #-------------------------------------------------------------------------
-from gi.repository import GObject, Gtk, Gdk
+from gi.repository import Gtk, Gdk, GLib
 
+#-------------------------------------------------------------------------
+#
+# Gramps modules
+#
+#-------------------------------------------------------------------------
+from ..utils import match_primary_mask
 #-------------------------------------------------------------------------
 #
 # InteractiveSearchBox class
@@ -45,7 +51,7 @@ from gi.repository import GObject, Gtk, Gdk
 #-------------------------------------------------------------------------
 
 
-class InteractiveSearchBox():
+class InteractiveSearchBox:
     """
     Mainly adapted from gtktreeview.c
     """
@@ -100,10 +106,7 @@ class InteractiveSearchBox():
         self._search_entry.disconnect(popup_menu_id)
 
         # Intercept CTRL+F keybinding because Gtk do not allow to _replace_ it.
-        default_accel = obj.get_modifier_mask(
-            Gdk.ModifierIntent.PRIMARY_ACCELERATOR)
-        if ((event.state & (default_accel | Gdk.ModifierType.CONTROL_MASK))
-                == (default_accel | Gdk.ModifierType.CONTROL_MASK)
+        if (match_primary_mask(event.state)
                 and event.keyval in [Gdk.KEY_f, Gdk.KEY_F]):
             self.__imcontext_changed = True
             # self.real_start_interactive_search(event.get_device(), True)
@@ -124,8 +127,8 @@ class InteractiveSearchBox():
     def _preedit_changed(self, im_context, tree_view):
         self.__imcontext_changed = 1
         if(self._entry_flush_timeout):
-            GObject.source_remove(self._entry_flush_timeout)
-            self._entry_flush_timeout = GObject.timeout_add(
+            GLib.source_remove(self._entry_flush_timeout)
+            self._entry_flush_timeout = GLib.timeout_add(
                 self._SEARCH_DIALOG_TIMEOUT, self.cb_entry_flush_timeout)
 
     def ensure_interactive_directory(self):
@@ -140,7 +143,7 @@ class InteractiveSearchBox():
             self._search_window.set_screen(screen)
             return
 
-        self._search_window = Gtk.Window(Gtk.WindowType.POPUP)
+        self._search_window = Gtk.Window(type=Gtk.WindowType.POPUP)
         self._search_window.set_screen(screen)
         if toplevel.has_group():
             toplevel.get_group().add_window(self._search_window)
@@ -222,8 +225,8 @@ class InteractiveSearchBox():
         self._renew_flush_timeout()
         # renew search timeout
         if self._entry_launchsearch_timeout:
-            GObject.source_remove(self._entry_launchsearch_timeout)
-        self._entry_launchsearch_timeout = GObject.timeout_add(
+            GLib.source_remove(self._entry_launchsearch_timeout)
+        self._entry_launchsearch_timeout = GLib.timeout_add(
             self._SEARCH_DIALOG_LAUNCH_TIMEOUT, self.search_init)
 
     def search_init(self):
@@ -236,10 +239,12 @@ class InteractiveSearchBox():
             return
 
         model = self._treeview.get_model()
+        if not model:
+            return
         selection = self._treeview.get_selection()
         # disable flush timeout while searching
         if self._entry_flush_timeout:
-            GObject.source_remove(self._entry_flush_timeout)
+            GLib.source_remove(self._entry_flush_timeout)
             self._entry_flush_timeout = 0
         # search
         # cursor_path = self._treeview.get_cursor()[0]
@@ -252,8 +257,8 @@ class InteractiveSearchBox():
 
     def _renew_flush_timeout(self):
         if self._entry_flush_timeout:
-            GObject.source_remove(self._entry_flush_timeout)
-        self._entry_flush_timeout = GObject.timeout_add(
+            GLib.source_remove(self._entry_flush_timeout)
+        self._entry_flush_timeout = GLib.timeout_add(
             self._SEARCH_DIALOG_TIMEOUT, self.cb_entry_flush_timeout)
 
     def _move(self, up=False):
@@ -268,7 +273,7 @@ class InteractiveSearchBox():
         selection = self._treeview.get_selection()
         # disable flush timeout while searching
         if self._entry_flush_timeout:
-            GObject.source_remove(self._entry_flush_timeout)
+            GLib.source_remove(self._entry_flush_timeout)
             self._entry_flush_timeout = 0
         # search
         start_count = self.__selected_search_result + (-1 if up else 1)
@@ -314,7 +319,7 @@ class InteractiveSearchBox():
         menu.connect("hide", self._enable_popdown)
 
     def _enable_popdown(self, obj):
-        self._timeout_enable_popdown = GObject.timeout_add(
+        self._timeout_enable_popdown = GLib.timeout_add(
             self._SEARCH_DIALOG_TIMEOUT, self._real_search_enable_popdown)
 
     def _real_search_enable_popdown(self):
@@ -351,7 +356,7 @@ class InteractiveSearchBox():
         # Launch search
         if (event.keyval in [Gdk.KEY_Return, Gdk.KEY_KP_Enter]):
             if self._entry_launchsearch_timeout:
-                GObject.source_remove(self._entry_launchsearch_timeout)
+                GLib.source_remove(self._entry_launchsearch_timeout)
                 self._entry_launchsearch_timeout = 0
             self.search_init()
             retval = True
@@ -388,10 +393,10 @@ class InteractiveSearchBox():
             self._search_entry.disconnect(self._search_entry_changed_id)
             self._search_entry_changed_id = 0
         if self._entry_flush_timeout:
-            GObject.source_remove(self._entry_flush_timeout)
+            GLib.source_remove(self._entry_flush_timeout)
             self._entry_flush_timeout = 0
         if self._entry_launchsearch_timeout:
-            GObject.source_remove(self._entry_launchsearch_timeout)
+            GLib.source_remove(self._entry_launchsearch_timeout)
             self._entry_launchsearch_timeout = 0
         if self._search_window.get_visible():
             # send focus-in event
@@ -492,7 +497,7 @@ class InteractiveSearchBox():
         """
         Search among the currently set search-column for a cell starting with
         text
-        It assumes that this column is currently sorted, and as 
+        It assumes that this column is currently sorted, and as
         a LIST_ONLY view it therefore contains index2hndl = model.node_map._index2hndl
         which is a _sorted_ list of (sortkey, handle) tuples
         """
